#include <string.h>
#include <stdio.h>

#include "dstring.h"

void DString::copy(const DString& b)
{
	if (sp)
	{
		delete sp;
		sp=0;
	}
	if (!b.IsEmpty())
	{
		sp = new char[b.getLen()+1];
		strcpy(sp, b.asChars());
	}
}

DString::DString()
{
	sp=0;
}

DString::DString(const DString& b)
{
	sp=0;
	if (!b.IsEmpty()) copy(b);
}

DString::DString(const char* s)
{
	if (s)
	{
		sp = new char[strlen(s)+1];
		strcpy(sp, s);
	}
	else sp=0;
}

DString::~DString()
{
	if (sp) delete sp;
}

DString::operator const char*()
{
	return asChars();
}

DString& DString::operator=(const DString& b)
{
	copy(b);
	return *this;
}

DString& DString::operator=(const char* s)
{
	if (sp)
	{
		delete sp;
		sp=0;
	}
	if (s)
	{
		sp = new char[strlen(s)+1];
		strcpy(sp, s);
	}
	return *this;
}

DString DString::operator+(const DString& b)
{
	char*tmp=0;
	if ((!IsEmpty()) || (!b.IsEmpty())) tmp = new char[getLen()+b.getLen()+1];
	if (!IsEmpty())
	{
		strcpy(tmp, sp);
		if (!b.IsEmpty()) strcat(tmp, b.asChars());
	}
	else if (!b.IsEmpty()) strcpy(tmp, b.asChars());
	DString temp(tmp);
	if (tmp) delete tmp;
	return temp;
}

DString DString::operator+(char*s)
{
	DString temp(s);
	return *this+temp;
}

int DString::operator==(const DString& b)
{
	if ((IsEmpty()) && (b.IsEmpty())) return 1;
	if ((!IsEmpty()) && (!b.IsEmpty())) if (strcmp(sp, b.asChars())==0) return 1;
	return 0;
}

int DString::operator==(const char*s)
{
	DString temp(s);
	return (*this==temp);
}

int DString::operator!=(const char*s)
{
	DString temp(s);
	return (!(*this==temp));
}

int DString::operator!=(const DString& b)
{
	return (!(*this==b));
}

DString& DString::operator+=(const DString& b)
{
	*this=*this+b;
	return *this;
}

char& DString::operator[] (int n)
{
	static char refchar;
	refchar=0;
	if ((n>=0) && (n<getLen()))	return sp[n];
	return refchar;
}

char* DString::asChars() const
{
	if (sp) return sp;
	return '\0';
}

int DString::asInt() const
{
	int tmp=0;
	if (sp) sscanf(sp, "%d", &tmp);
	return tmp;
}

int DString::empty()
{
	if (sp)
	{
		delete sp;
		sp=0;
		return 1;
	}
	return 0;
}

int DString::IsEmpty() const
{
	if (!getLen()) return 1;
	return 0;
}

double DString::asDouble() const
{
	double tmp=0;
	if (sp) sscanf(sp, "%lf", &tmp);
	return tmp;
}

int DString::getLen() const
{
	if (sp) return strlen(sp);
	return 0;
}

DString DString::IsString(char* s) const
{
	char*tmp=0;
	if ((sp) && (s)) tmp=strstr(sp, s);
	if (tmp)
	{
		char*tmp2 = new char[strlen(s)+1];
		strncpy(tmp2,tmp,strlen(s));
		tmp2[strlen(s)]=0;
		DString temp(tmp2);
		delete tmp2;
		return temp;
	}
	DString temp;
	return temp;
}

DString DString::GetString(int a, int b) const
{
	DString temp(*this);
	temp=temp.TrimLeft(a);
	temp=temp.TrimRight(temp.getLen()-b+a-1);
	return temp;
}

DString DString::TrimRight(int x) const
{
	DString temp;
	if ((x>=0) && (x<=getLen()))
	{
		char*tmp = new char[getLen()-x+1];
		strncpy(tmp, sp, getLen()-x);
		tmp[getLen()-x]=0;
		temp=tmp;
		delete tmp;
	}
	return temp;
}

DString DString::TrimLeft(int x) const
{
	DString temp;
	if ((x>=0) && (x<=getLen())) temp=&sp[x];
	return temp;
}
